#!/usr/bin/env fish
#
# Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved
#

function ___virtualenv_create -d "Creates/updates the python virtualenv"
    set --local options 'c/crypto' 'n/noisy' 'u/update'
    argparse $options -- $argv
    or begin; ___error "\nIncorrect arguments for 'environs.fish': $argv"; exit 1; end

    set -l pip_verbose "-q"
    if set -q _flag_noisy
        set pip_verbose "-v"
    end
    if set -q _flag_update
        set use_upgrade "--upgrade"
        set message "update"
    end

    if not test -d ~/.virtualenvs/HELLO
        which python3 >/dev/null
        if [ $status -ne 0 ]
            echo "$(which python3; echo $status)"
            ___error "Python 3.x is required"
            exit 0
        end

        ___action "CREATING 'HELLO' virtualenv ..."
        python3 -m venv ~/.virtualenvs/HELLO
        source ~/.virtualenvs/HELLO/bin/activate.fish

        # Make sure we draw from our internal server, which mirrors the public PyPi
        pip config set global.index-url https://artifactory-qa.cbenglab.com/artifactory/api/pypi/pypi-virtual/simple

        pip install $pip_verbose --upgrade pip
        pip install  $pip_verbose --upgrade pip-tools
        rm -rf ~/.cache/pip
        rm -rf ~/Library/Caches/pip

        if [ -f $HELLO_ROOT/test/environment/.requirements_hash ]
            rm -rf $HELLO_ROOT/test/environment/.requirements_hash
        end
    else
        ___action "USING 'HELLO' virtualenv ..."
        source ~/.virtualenvs/HELLO/bin/activate.fish

        if set -q _flag_update
            ___action2 " ... and updating"

            pip install -q --upgrade pip
            pip install -q --upgrade pip-tools
            rm -rf $HELLO_ROOT/test/environment/.requirements_hash
        end
    end

    # handle changes to the python requirements, make sure we have an exported one for this shell
    if test -f $HELLO_ROOT/test/environment/requirements.txt
        set hash (md5sum $HELLO_ROOT/test/environment/requirements.txt | awk '{print $1}')

        set -l oldhash ""
        if test -f $HELLO_ROOT/test/environment/.requirements_hash
            set oldhash (cat $HELLO_ROOT/test/environment/.requirements_hash)
        end

        if test -f $HELLO_ROOT/test/environment/.crypto_override
            set -x CRYPTOGRAPHY_SUPPRESS_LINK_FLAGS 1
            set -x LDFLAGS "$(brew --prefix openssl@1.1)/lib/libssl.a $(brew --prefix openssl@1.1)/lib/libcrypto.a"
            set -x CFLAGS "-I$(brew --prefix openssl@1.1)/include"
        end

        if [ "$oldhash" != "$hash" ]
            ___action2 " ... update requirements.txt"
            pip install $pip_verbose -I --trusted-host artifactory-pub.bit9.local --use-pep517 -r $HELLO_ROOT/test/environment/requirements.txt
            or begin; ___error "$Unable to install requirements.txt!"; return 1; end
            echo $hash > $HELLO_ROOT/test/environment/.requirements_hash
        else
            ___action2 " ... requirements.txt ok"
        end
    end

    set -l check "*$HELLO_ROOT/test/pytest/lib*"
    switch "$PYTHONPATH"
        case "$check"
            ___action2 " ... PYTHONPATH ok"
        case "*"
            set -gx PYTHONPATH ".:$PYTHONPATH:$HELLO_ROOT/test/pytest/lib"
            ___action2 " ... updating PYTHONPATH"
    end

      set -l check "*$HELLO_ROOT/test/utilities*"
      set -e found
      for arg in $PATH
          switch $arg
              case $check
                  ___action2 " ... PATH ok"
                  set found $arg
          end
      end
      if not set -q found
          set -x PATH $HELLO_ROOT/test/utilities $PATH
          ___action2 " ... updating PATH"
      end
end


function ___virtualenv_remove
    if test -d ~/.virtualenvs/HELLO
        deactivate
        ___action "REMOVING 'hinting' virtualenv ..."
        rm -rf ~/.virtualenvs/HELLO
    end
end
