#!/usr/bin/env fish
#
# Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved
#
function ,hello
    cd $HELLO_ROOT
end

function +coverage
    pushd $HELLO_ROOT > /dev/null

    go test -v -race -covermode=atomic -coverpkg=./cmd,./internal/... -coverprofile=coverage.out ./internal/... 2>&1 > /dev/null
    go tool cover -html=coverage.out -o=coverage.html
    set -l pct (go tool cover -func coverage.out | grep "total:" | awk '{print $3}')
    echo; ___notice "Coverage for './cmd,./internal/...' is $pct"

    popd > /dev/null
end

function +configs
    set -l zz (ls -1 $SHC | grep -v README.md)
    echo
    for i in $zz; ___action $i; end
    echo
end

function +app
    set --local options 'h/help' 'k/kill' 'n/notail'
    argparse $options -- $argv
    or ___error "See '+app --help' for usage"; or return 1

    if set -q _flag_help
        set_color $ORIENT_HELP_CODE -o
          echo
          echo "+app [OPTIONS]"
        set_color normal; set_color $ORIENT_HELP_CODE
          echo "================================================================================"
        set_color normal; set_color $ORIENT_HELP_CODE2
          echo "Start the hinting service in a local kubernetes cluster."
          echo
          echo "  -h | --help"
          echo "    Print this help"; echo
          echo
          echo "  -k | --kill"
          echo "    Kill off the existing hello service"; echo
        set_color normal
        return 0
    end

    if set -q _flag_kill
        helm uninstall ems-device-hello -n local-apps
    else
        pushd $HELLO_ROOT
        make local-app-start-notail
        or begin; ___error "Unable to start local app"; popd; return 1; end
        popd
    end
end

function +infra
    set --local options 'd/delete' 'h/help' 'u/update'
    argparse $options -- $argv
    or ___error "See '+infra --help' for usage"; or return 1

    if set -q _flag_help
        set_color $ORIENT_HELP_CODE -o
          echo
          echo "+infra [OPTIONS]"
        set_color normal; set_color $ORIENT_HELP_CODE
          echo "================================================================================"
        set_color normal; set_color $ORIENT_HELP_CODE2
          echo "Start the hello infrastructure using helm."
          echo
          echo "  -h | --help"
          echo "    Print this help"; echo; echo

          echo "Options:"
          echo "  -d | --delete"
          echo "    Delete local namespaces"; echo
          echo "  -u | --update"
          echo "    Update helm files before deployment"; echo
        set_color normal
        return 0
    end

    pushd $HELLO_ROOT >/dev/null

    if set -q _flag_locks
        ___warning "$Updating helm ..."
        make update-helm
    end

    set -gx ENV local
    if set -q _flag_delete
        make delete-namespaces
    end

    make local-infra-start-notail
    or begin; ___error "Unable to start local infrastructure"; popd; return 1; end

    popd  >/dev/null
end
