#  coding: utf-8
#  Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved
################################################################################

import logging
from typing import Optional, Dict, Any, Union

from lib.models.base_model import BaseModel
from .base_connect import BaseConnect

_logger = logging.getLogger(__name__)

# routing basics
ROOT = "/services"
V1 = f"{ROOT}"


class HelloConnect(BaseConnect):
    """
    Class to handle rest calls that configure or modify the hinting service.
    """

    @staticmethod
    def _convert_data(source: Any) -> Any:
        """Implement model or dict as data for API call."""
        if isinstance(source, BaseModel):
            return source.to_dict()  # actual data structure
        elif isinstance(source, Dict):
            return source
        else:
            _logger.debug(f"NOTE: Using {type(source)} for converted data!")
            return source  # raw dict or other data for negative testing

    @staticmethod
    def _get_field(source: Union[Dict, BaseModel], key: str) -> Optional[Any]:
        """Get data field from model or dict, or nothing if anything else."""
        if isinstance(source, BaseModel):
            return getattr(source, key)  # get from model
        elif isinstance(source, Dict):
            return source.get(key)  # get from dict
        else:
            # noinspection PyBroadException
            try:
                return getattr(source, key)  # get from object, if possible
            except Exception:
                return None  # no field to get

    # ----- Endpoints --------------------

    # ----- Miscellaneous Endpoints --------------------

    def health_check_get(self) -> bool:
        """
        Check health of the service.

        :return: True if health ok
        """
        # noinspection PyBroadException
        try:
            self._get(f"{ROOT}/healthCheck")
            return self.last_status == 200
        except Exception as err:
            _logger.debug(f"Healthcheck saw {err}")
            return False
