#!/usr/bin/env bash
#
# Copyright (c) 2023 VMware Carbon Black, all rights reserved
#

# NOTE: This script is assumed to be run from the repository root!

_dbg=0
_no_service=0
_run_mock=0
_local=0
while getopts "dlms" options; do
    case "${options}" in
        d) _dbg=1;;
        l) _local=1;;
        m) _run_mock=1;;
        s) _no_service=1;;
    esac
done

# handle -d (debugging) flag
if [[ ${_dbg} -eq 1 ]]; then
    set -x
fi

if [[ ${_local} -eq 1 ]]; then
    # Used for local use of the functional tests
    export AWS_ENDPOINT=http://localhost:4566
    export REGION=us-east-1
    export AWS_ACCESS_KEY_ID=test
    export AWS_SECRET_ACCESS_KEY=test
    export LCT_STREAM_NAME=local-lct-stream

    export IS_LOCAL_TEST=true
    export IS_USING_MOCK=true
    export LOG_LEVEL=debug
fi

# This script is designed for pipeline use so you will need to use the -s option if running locally against the
# service if it is running in kubernetes.
_service=0
if [[ ${_no_service} -eq 1 ]]; then
    go run ./cmd/main.go -profiler &
    _service=$!
    echo "Hello service started locally with pid ${_service} ..."
else
    echo "No hello service started -- presumed to be handled externally"
fi

# Wait for dependent services to come up
wait_for_curl () {
  local TIMEOUTS=0
  local URL=$1
  local SERVICE_NAME=$2
  until curl -Ss -fv "${URL}" || [[ ${TIMEOUTS} -eq 12 ]]; do
    echo "Waiting for ${SERVICE_NAME} to come up...";
    sleep 5;
    ((TIMEOUTS+=1))
  done
  STATUS_CODE=$(curl --write-out %{http_code} --silent --output /dev/null "${URL}")
  if [[ "${STATUS_CODE}" != "200" ]]; then
    docker logs ${SERVICE_NAME}
    printf "\n${SERVICE_NAME} returned a %s status code" "${STATUS_CODE}. See docker logs above"
    exit 1
  fi
}

wait_for_curl "http://localhost:5004/hello-api/healthCheck?isReady" "hello"

# Execute any analysis utilities or cleanup
function on_exit {
    if [[ ${_pid} -ne 0 ]]; then
        kill $_pid
    fi
    if [[ ${_service} -ne 0 ]]; then
        kill $_service
    fi
    echo "DONE!"
}
trap on_exit EXIT


echo ""
echo "################################################################################"
echo "                        Service ready, begin testing"
echo "################################################################################"
echo ""

# FUTURE: Disabled for now, may be removed in the future, if SensorSim not used
# Run the functional tests after we update any requirements
#pip install --upgrade -I --trusted-host artifactory-pub.bit9.local --use-pep517 -q -r ./test/environment/requirements.txt
#pushd ./test/pytest
#pytest --dbg -m "not utility" --junitxml ./report.xml
#popd

export LOG_LEVEL="debug"
export ENV_CATEGORY="local"

go test -v ./test/functional/...
