<!--suppress CssUnusedSymbol, SpellCheckingInspection-->
<style>
  .blue { color: dodgerblue; }
  .op {  border-bottom: thin solid grey; font-family: monospace; color: chocolate; margin-left:25px; width: fit-content}
  .sep { height: 2px; border-width: 0; background-color: goldenrod; }
  .text { padding: 2px 3px 2px 3px; border: thin dotted white; background-color: black; color: cadetblue; margin-left:25px;  width: fit-content}
  h1 { color: chocolate; }
</style>

# Python Utilities
This folder contains various python utilities that can be easily invoked on the command line when the <b class="blue">ORIENT</b> environment
is active (when sourced, the HELLO python environment is enabled with all requirements and this folder is added
to the PATH environment variable).

> _**NOTE:** These utilities assume that you are using the virtualenv created by the
> <b class="blue">ORIENT</b> environment, see
> [**ORIENT.MD**](../../documentation/ORIENT.md) for more information._


<hr class="sep">

## 1. send_hello.py
This utility is used to simulate a sensor hello call to a specified environment. A run session will create the device information, 
submit one or more hello requests, then perform a device removal call.

To use this tool with certain environments (dev01 and test03), you must have defined in your environment the variables 
`MY_CSR_ADMIN` with your CSR Login and `MY_CSR_PASSWORD` with your CSR password.

Before using the utility, source the desired environment shell:

| Environment | Command                                                | Notes        |
|-------------|--------------------------------------------------------|--------------|
| DEV01       | `source $HELLO_ROOT/test/utilities/config/dev01.sh`    |              |
| TEST03      | `source $HELLO_ROOT/test/utilities/config/test03.sh`   |              |
| KCDEV01     | `source $HELLO_ROOT/test/utilities/config/kcdev01.sh`  | _(Mosaic 2)_ |
| INT1USW2    | `source $HELLO_ROOT/test/utilities/config/int1usew.sh` | _(Mosaic 2)_ |
| INT2USE2    | `source $HELLO_ROOT/test/utilities/config/int2use2.sh` | _(Mosaic 2)_ |

> _**NOTE:** If you have sourced <b class="blue">ORIENT</b>, you can use `source $SHC/<filename>` instead of the long form.
> Above shells assume bash/zsh; if using the fish shell, source the ".fish" version of the scripts._

If executing within intellij, add the desired configuration file as an envfile.

### 1.1. Options
The following command line options are available:
* `--age <int>`: Allows you to define the time in minutes since the last update and last live query before you execute the first hello call.
* `--count <int>`: Allows you to specify the number of hello requests for the given session.  If not supplied, defaults to 1.
* `--debug`: Turns on debug logging level for this utility as well as from PythonSensorSim.
* `--delay <int>`: Allows you to specify the time delay between multiple hello requests, in seconds.  If not supplied, defaults to 10 seconds. Only applies when --count is greater than 1.  A value of 0 will produce no delay between requests.
* `--deploy <str>`: Allows you to specify the deployment type as one of 'endpoint' or 'workload'.  If not supplied, defaults to 'endpoint'.
* `--info`: Dump information about the connected user and the simulated sensor.
* `--prefix <str>`: Prepends the provided string to the hostname of the sensor and the email of the user.
* `--stats`: Calculates statistics to be displayed at the end of the run.
* `--type <str>`: Allows you to specify the device type as one of 'windows', 'mac' or 'linux'.  If not supplied, defaults to 'windows'.

### 1.2. Output
Sample output of a "run once" execution (default count):
<pre class="text">
/Users/bchoinski/.virtualenvs/HELLO/bin/python /Users/bchoinski/dev/hello-api/test/utilities/send_hello.py 
INFO:psc_auth_integration.entities.auth_session:Login #1 for username "enkitest@vmware.com"
INFO:psc_auth_integration.entities.auth_session:Login response: 200: {"designPartner":false,"locale":null,"currentOrgId":1,"currentOrgKey":"J7G6DTLN","loginId":6324499,"loginName":"enkitest@vmware.com","loginStatus":"SUCCESS","firstName":"EnkiTest","lastName":"QA","analyticsProfile":"DOME","analyticsVersion":"V2","flagged":false,"uiProfile":"CONFER_FULL","orgRoles":{"1":"DEPRECATED","3978809":"DEPRECATED","1607198":"DEPRECATED","35593":"DEPRECATED","2679585":"DEPRECATED","6889445":"DEPRECATED","4320846":"DEPRECATED","1047321":"DEPRECATED","88728":"DEPRECATED","2800051":"DEPRECATED","2295737":"DEPRECATED","2859189":"DEPRECATED","4271582":"DEPRECATED","6874824":"DEPRECATED","1287985":"DEPRECATED","1889976":"DEPRECATED","1989691":"DEPRECATED","2704527":"DEPRECATED","1044264":"DEPRECATED","6873792":"DEPRECATED","1923756":"DEPRECATED","6443217":"DEPRECATED","3710476":"DEPRECATED","5961070":"DEPRECATED","80180":"DEPRECATED","1952935":"DEPRECATED","2084003":"DEPRECATED","1858652":"DEPRECATED","3153139":"DEPRECATED","1992535":"DEPRECATED","1482":"DEPRECATED","37324":"DEPRECATED","6085520":"DEPRECATED","1241052":"DEPRECATED","3357026":"DEPRECATED","1781193":"DEPRECATED","33753":"DEPRECATED","2405062":"DEPRECATED","1406449":"DEPRECATED","283105":"DEPRECATED","2255303":"DEPRECATED","215397":"DEPRECATED","3722193":"DEPRECATED","47979":"DEPRECATED","2072179":"DEPRECATED","1011":"DEPRECATED","1020":"DEPRECATED","2402265":"DEPRECATED","1662822":"DEPRECATED"},"orgNames":{"1":"confer.net","3978809":"gauravtest1.org","1607198":"vuln-assessment.cwp.com","35593":"00000.com","2679585":"ngav.org","6889445":"cydeletableorganizationcaea8547.com","4320846":"psc_access_integration-auto-eee3a8f8-8d73-4365-9911-0b90064b3eee.auto","1047321":"bulk-upgrade-test.com","88728":"aobo-audit.com","2800051":"cwp.guru.org","2295737":"rgschild.org","2859189":"cypress-dashboard-v2.org","4271582":"rapid-configs.org","6874824":"klim.child_1.org","1287985":"kim-test","1889976":"origami-device-control-dev.com","1989691":"cqetest.org","2704527":"cbc-prevention.com","1044264":"sam2020","6873792":"klim.child_2.org","1923756":"cwp-regression-01.com","6443217":"uae.com","3710476":"cb.public.cloud.dev","5961070":"hbfw.org","80180":"saPerms.org","1952935":"vuln-test.cwp.com","2084003":"orggroup222.com","1858652":"amsiDefenseElsie.com","3153139":"fp.test.dashboard.io","1992535":"test01.net","1482":"samueldelesque.net","37324":"5thStreetCoffee9.org","6085520":"hbfw-dev-testing.com","1241052":"IONsandbox","3357026":"demo.cwp.org","1781193":"joash_demo_47.org","33753":"saLiveResponse.org","2405062":"gauravtest.org","1406449":"cwp","283105":"internal-ui-automation-mssp.org","2255303":"BhaskarTest.org","215397":"aobo-pscr","3722193":"test-parvez.org","47979":"pscr-test-01","2072179":"CWP-demo.com","1011":"e-ie.io","1020":"mrtest.org","2402265":"core-prevention.org","1662822":"modaka.test"},"uiFeatureFlags":{"unifiedPlatformOnboarding":true,"endpointsVulnerabilityAssessment":true,"tempCISBenchmarkV2":true,"disableNotifications":false,"liveQueryAutoRun":false,"nestedEasySearch":true,"sensorThrottling":true,"cbDefense":true,"autoUpdateSensors":true,"elsieSearchTranslation":true,"sensorUpdateServices":true,"pscResponse":true,"tempEmsHintingServicePhaseTwo":false,"alertClassification":false,"pendoAnalytics":true,"kubernetesSecurityGuardrails":false,"pscLCDataFork":true,"elsieEndShortTermES":true,"rolesEditV2":true,"tempAssetGroupsNext":true,"threatSight":true,"liveQuerySearch":true,"corePreventions":true,"mdrMagagement":true,"threatResearchCmd":true,"kubernetesSecurityImageScanning":false,"labs":false,"triageElsification":true,"exportInvestigateV1":true,"conferAV":true,"investigateEventsSubtabs":true,"tempSecureState":true,"tempNsxRemediation":true,"assetGroupsManagement":true,"pscModules":false,"CONFER_PROTECTION":true,"tempMdr":false,"exportInvestigateV2":true,"deviceControl":true,"virusTotal":true,"cbDefenseElsie":true,"controlPanel":true,"policyCopy":true,"liveQueryPanel":true,"notifications":true,"tempDismissVulnerabilityAssessment":true,"emsDeviceAuth":false,"kubernetesSecurityDemo":false,"tempMdrManagement":true,"showNgav":true,"orgDetailsProvisioning":true,"kubernetesSecurityRuntimeProtection":false,"backgroundScan":false,"hostBasedFirewall":false,"thubsRuleMgmtV2":true,"publicCloud":false,"deviceControlMacOS":false,"tempEventForwarderVersion":true,"policiesV2":true,"saml":true,"liveResponseV6":true,"backendEmsHelloMsgs":true,"googleAnalytics":true,"leftNavRedesign":true,"analyzeUpload":true,"threatHunterPlusDefenseEventsSearch":true,"filteredSensorKits":true,"accessProfiles":true,"deviceControlPolicyBlocking":true,"mitreTTPs":true,"ngav":false,"confirmExternalLinks":false,"disableEntitlements":false,"liveQueryFeaturesThatRequireMosaic":true,"tableColumnCustomization":true,"massSensorMgmt":false,"malwareAutoDelete":true,"passwordProtectedUninstall":true,"ORG_GROUPS":false,"executiveReportExport":false,"newSensorControls":true,"updateSensors":true,"investigateCollapseWarning":true,"investigateRecentSearches":false,"newUserGuide":true,"managedDetectionForEEDR":true,"reateOrgWithEntitlements":true,"tempKubernetesClusterScanning":false,"amsiPrevention":false,"vulnerabilityAssessmentTemp":true,"tempAssetGroupsManagement":false,"notifications_user_alerts":true,"createOrgWithEntitlements":true,"policyActionOverride":true,"critsConnector":true,"queueDefenseToTHConversion":true,"kubernetesSecurityRuntimeAdvanced":false,"VdiSupport":true,"configureDefensePerPolicy":false,"tauTinWidget":true,"matomoAnalytics":true,"isOrgVersion3":true,"msspOrgGroups":false,"tempAuthEvents":true,"olesEdit":false,"disableIamOrgCreation":false,"ENABLE_AV_SERVER_HOTFIX":true,"androidDownload":true,"customConnectors":true,"emsDeviceAuthEnforcement":true,"kubernetesSecurityRuntimeEssentials":false,"lc65ResultCounts":false,"liveQueryMacOS":true,"apcSettings":true,"malwareRemoval":true,"allowNgav":false,"msspSwitch":true,"unifiedAlerting":true,"variableDataRetention":false,"orgGroups":false,"autoUpdate":true,"threatHunterMacOS":true,"allowUninstall":true,"tempAssetRegistry":false,"analyzeTreeNodeCollapse":true,"managedDetectionAlertDigest":true,"eventForwarderConfig":true,"internalSensorSettings":true,"tempDataForwarderWatchlistHit":true,"releaseNotes":false,"recommendations":true,"tempEmsHintingServicePhaseOne":true,"investigateEventTimeline":false,"elsiePOC":true,"liveQueryLinuxOS":true,"tempEventForwarderAdvancedFiltering":true,"liveQuery":true,"osxDownload":true,"sensorV3.3":true,"backgroundScanStatus":false,"ws1IntegrationSensorUrls":true,"CONFER_FULL":true,"vulnerabilityAssessment":true,"packetCapture":true,"deviceControlAlerts":true,"sensorV3.5":true,"workloadsManagement":true,"deviceTagging":false,"customerRoles":true,"vmwAppDefense":false,"emsDeviceCrud":false,"tempMdrNotifications":true,"xdr":true,"managedDetectionResponse":true,"tempDeobfuscationv2":true,"tempWindowsArm":true,"tempMDRThreatHunting":true,"threatGraph":true,"tempCSRDeprovisioning":true,"tempSensorUpgradePage":true,"mixpanelAnalytics":true},"message":"Success","success":true}
{'Date': 'Thu, 17 Aug 2023 16:46:35 GMT', 'Content-Type': 'application/json; charset=utf-8', 'Content-Length': '7026', 'Connection': 'keep-alive', 'Server': 'gunicorn', 'Set-Cookie': 'SESSION=b2cf72b5-a91d-4465-b542-6467bb10d7cc; Secure; HttpOnly; Path=/', 'X-Csrf-Token': '7ca5909b4c318a6149822f72704f42f72db9dc7cf66f3886f8132b94243f3f3f', 'X-Processed-By': 'psc.auth'}
INFO:psc_auth_integration.entities.auth_session:Refresh user info: https://defense-dev01.cbdtest.io/userInfo
INFO:__main__:Authorizing with user `enkitest@vmware.com` against https://defense-dev01.cbdtest.io
INFO:psc_server_support.api.app_services_api.app_services_base:Getting company registration code for org 1
INFO:psc_server_support.api.app_services_api.app_services_base:Got company registration code E7TD1QQA
INFO:psc_server_support.api.app_services_api.app_services_base:Got encoded company registration code 2ICWP88KHMSZYMSZ4FB1UMWSAWB@H
Attempting to create a new request of type REQ_REGISTER_CONTROL (4097). Proto Version is 18.
INFO:psc_protocol.request_factory:Attempting to create a new request of type REQ_REGISTER_CONTROL (4097). Proto Version is 18.
Unpacked Message Code: MessageCode.RESP_REGISTER_CONTROL; Protocol Version: 18
INFO:psc_protocol.response_parser:Unpacked Message Code: MessageCode.RESP_REGISTER_CONTROL; Protocol Version: 18
INFO:python_sensor_sim.python_sensor_sim:Successfully registered device with registration-id: 1-19109240
INFO:root:===== Created ==========
Sensor: Hostname cbd-internal-auto-8fa86f4f-b29b-48a5-a07c-63697440bd54, Orgkey J7G6DTLN (env dev01, id 1)
           DeploymentType: DeploymentType.ENDPOINT
           DeviceType:     SensorDeviceType.WINDOWS
           Last Update:    0  (1970-01-01 00:00:00)
           Live Query:     None

Attempting to create a new request of type REQ_HELLO (8220). Proto Version is 18.
INFO:psc_protocol.request_factory:Attempting to create a new request of type REQ_HELLO (8220). Proto Version is 18.
Unpacked Message Code: MessageCode.RESP_HELLO; Protocol Version: 18
INFO:psc_protocol.response_parser:Unpacked Message Code: MessageCode.RESP_HELLO; Protocol Version: 18
INFO:root:===== After Hello ==========
Sensor: Hostname cbd-internal-auto-8fa86f4f-b29b-48a5-a07c-63697440bd54, Orgkey J7G6DTLN (env dev01, id 1)
           DeploymentType: DeploymentType.ENDPOINT
           DeviceType:     SensorDeviceType.WINDOWS
           Last Update:    1692269721193  (2023-08-17 10:55:21.193000)   <<< Updated
           Live Query:     1692290714546  (2023-08-17 16:45:14.546000)   <<< Updated
           Hints: SI_HINT_NEW_POLICY
                  SI_HINT_NEW_CONFIG_DATA
                  SI_HINT_HASH_LIST_REFRESH
                  SI_HINT_ZIP_CONTAINER
                  SI_HINT_UNINSTALL_CODE
                  SI_HINT_LIVE_QUERY
                  SI_HINT_BLADE_ACTION

INFO:psc_server_support.api.app_services_api.app_services_base:Sending request to uninstall devices 19109240
INFO:psc_server_support.api.app_services_api.app_services_base:Getting device info for device 19109240 in org 1
INFO:psc_server_support.api.app_services_api.app_services_base:Deleting deregisted devices 19109240

Process finished with exit code 0
</pre>

With multiple calls you may well see new hints generated from the backend:
<pre class="text">
INFO:root:===== After Hello ==========
Sensor: Hostname cbd-internal-auto-c1b0c2c4-3366-4330-ad04-2680f4ea9432, Orgkey J7G6DTLN (env dev01, id 1)
           DeploymentType: DeploymentType.ENDPOINT
           DeviceType:     SensorDeviceType.WINDOWS
           Last Update:    1692269721193  (2023-08-17 10:55:21.193000)   <<< Updated
           Live Query:     1692290714546  (2023-08-17 16:45:14.546000)   <<< Updated
           Hints: SI_HINT_NEW_POLICY
                  SI_HINT_NEW_CONFIG_DATA
                  SI_HINT_HASH_LIST_REFRESH
                  SI_HINT_ZIP_CONTAINER
                  SI_HINT_UNINSTALL_CODE
                  SI_HINT_LIVE_QUERY
                  SI_HINT_BLADE_ACTION

Attempting to create a new request of type REQ_HELLO (8220). Proto Version is 18.
INFO:psc_protocol.request_factory:Attempting to create a new request of type REQ_HELLO (8220). Proto Version is 18.
Unpacked Message Code: MessageCode.RESP_HELLO; Protocol Version: 18
INFO:psc_protocol.response_parser:Unpacked Message Code: MessageCode.RESP_HELLO; Protocol Version: 18
INFO:root:===== After Hello ==========
Sensor: Hostname cbd-internal-auto-c1b0c2c4-3366-4330-ad04-2680f4ea9432, Orgkey J7G6DTLN (env dev01, id 1)
           DeploymentType: DeploymentType.ENDPOINT
           DeviceType:     SensorDeviceType.WINDOWS
           Last Update:    1692269721193  (2023-08-17 10:55:21.193000)
           Live Query:     None
           Hints: 

Attempting to create a new request of type REQ_HELLO (8220). Proto Version is 18.
INFO:psc_protocol.request_factory:Attempting to create a new request of type REQ_HELLO (8220). Proto Version is 18.
Unpacked Message Code: MessageCode.RESP_HELLO; Protocol Version: 18
INFO:psc_protocol.response_parser:Unpacked Message Code: MessageCode.RESP_HELLO; Protocol Version: 18
INFO:root:===== After Hello ==========
Sensor: Hostname cbd-internal-auto-c1b0c2c4-3366-4330-ad04-2680f4ea9432, Orgkey J7G6DTLN (env dev01, id 1)
           DeploymentType: DeploymentType.ENDPOINT
           DeviceType:     SensorDeviceType.WINDOWS
           Last Update:    1692291543608  (2023-08-17 16:59:03.608000)   <<< Updated
           Live Query:     None
           Hints: SI_HINT_NEW_POLICY
                  SI_HINT_NEW_CONFIG_DATA
</pre>

With the `--info` option you will get user

<pre class="text">

================================================================================
  USER: 5NUynncTaxQS2tDXkSULGyIoCqweMw5eC1K on https://kcdev01.cbc-eng.vmware.com
  ORG: EWRT35QWVM (id 6907921, name 'hello-service-test.com')
================================================================================

</pre>

and sensor information.
<pre class="text">

================================================================================
  HOSTNAME: enki-hello-f8a65a4a-bd48-472c-8863-cde90bac9811 (id 19117498)
  DEVICE: SensorDeviceType.WINDOWS, DEPLOYMENT: DeploymentType.ENDPOINT
  GROUP: None (id None)
================================================================================

</pre>

#### 1.2.1. Statistics
If the `--stats` option is used, run information is provided at the end:
<pre class="text">
===== Execution Statistics ========================================
  10 hello calls
    Last Update Time:    1694668238075 (1 changes)
    Last Livequery Time:             0 (0 changes)
</pre>

### 1.3. Observing Logs
Using `kubectl` or `kubectx`, set your context to the desired environment.  Use `kubectl` to find the hello pod:
<p class="op">
kubectl get pods | grep hello
</p>

Select the name of the "ems-device-hello" pod from the output
<pre class="text">
ems-device-hello--test                                            0/1     Completed                    0                  19h
<b style='color:red;'>ems-device-hello-579d5475fb-c6px5</b>                                 1/1     Running                      0                  19h
kc-hello-world-8648cd9779-sp68s                                   1/1     Running                      0                  5d12h
</pre>

Looking at the output of the utility, you can get the deviceid as the right part of the registration id:
<pre class="text">
INFO:python_sensor_sim.python_sensor_sim:Successfully registered device with registration-id: 6907921-<b style='color:red;'>19119040</b>
</pre>

And to get the logs for the device:
<p class="op">
kubectl logs <i>&lt;pod-name&gt;</i> | grep id:<i>&lt;dev-id&gt;</i>
</p>

## 2. weighting.py
Contacts AWS with connection information for the load balancers of the various production environments, returning
the weighting for the HELLO service in those environments:
<pre class="text">
Current mosaic2 weights for /services/hello for "PROD"
================================================================================
   EAP01      -- port 443:   0%, port 5443:   0%
   PROD01     -- port 443:  80%, port 5443:  80%
   PROD02     -- port 443:   0%, port 5443:   0%
   PROD05     -- port 443:   0%, port 5443:   0%
   PROD06     -- port 443:  40%, port 5443:  40%     -- (Frankfurt)
   PRODNRT    -- port 443:  80%, port 5443:  80%     -- (APNE1: Tokyo)
   PRODSYD    -- port 443:  80%, port 5443:  80%     -- (APSE2: Sydney)
================================================================================
</pre>

This utility will note a warning for any environment where the 443 and 5443 port weightings differ:
<pre class="text">
   PRODNRT    -- port 443:  80%, port 5443:  80%     -- (APNE1: Tokyo)
!  PRODSYD    -- port 443:  80%, port 5443:  90%     -- (APSE2: Sydney)
================================================================================
! indicates that 443 and 5443 weights differ
</pre>

This utility will also flag if the total for mosaic2 and cbd is NOT 100% for either port
<pre class="text">
   PROD06     -- port 443:  80%, port 5443:  80%     -- (aka Frankfurt)
                 \_443 cbd = 10% 
   PRODNRT    -- port 443:  80%, port 5443:  80%     -- (aka PRD1APNE1, Tokyo)
                 \_443 cbd = 10%  5443 cbd = 30%
</pre>

If an arn cannot be found (removed or renamed), you will see warnings and the weightings will have a value of -99%:
<pre class="text">
2023-10-13 15:45:58,016 - [42560] WARNING [__main__:149] Unknown TargetGroupArn: arn:aws:elasticloadbalancing:us-east-1:132308400445:targetgroup/prod02-cbd-devices-mcs2/5dba4f5de1e0a914
   PROD02     -- port 443: -99%, port 5443:  20%
</pre>

This utility requires local access to AWS -- you may need to run `gimme-aws-creds` first to set your local credentials.
Production requires access with the "scargo" account.

### 2.1. `--only` option
This allows you to get information for a specific environment.  If you provide one that does not exist you will be
presented with a list of available ones:
<pre class="text">
<b>hello-api</b> % weighting.py --only foobar        
***** For '--only', you must specify one of: ['EAP01', 'PROD01', 'PROD02', 'PROD05', 'PROD06', 'PRODNRT', 'PRODSYD']
</pre>

### 2.2. `--dev` option
To see information about development systems, you need to have aws creds with the "confer-test" account 
and use the `--dev` option:
<pre class="text">
Current mosaic2 weights for /services/hello for "DEV"
================================================================================
   DEV01      -- port 443:  80%, port 5443:  80%
   TEST03     -- port 443:  80%, port 5443:  80%
================================================================================
</pre>
